 #include <Wire.h>
#include "SSD1306Wire.h"
#include <Adafruit_ADS1015.h>
#include <KalmanFilter.h>
#include "images.h"
#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>

//---------------------IMU constants and variables-----------------//
int16_t accX, accY, accZ;
int16_t delt_aX, delt_aY, delt_aZ;

float Pitch, Roll;
boolean tiltInhibit = true;

uint8_t i2cData[14]; // Buffer for I2C data

const uint8_t IMUAddress = 0x68; // AD0 is logic low on the PCB
const uint16_t I2C_TIMEOUT = 1000; // Used to check for errors in I2C communication

//---------------Sharp Sensor constants and Variables-------------//
Adafruit_ADS1115 ads;
int16_t sharp_adc;
double voltage;
double distance;

unsigned long distancetimer;
const int timeinterval = 100;
const int sample_size_l = 17;
const int sample_size_s = 10;
double mean_l, mean_s, sd_l, sd_s;
double dist_vector[sample_size_l];
double kernel_l[sample_size_l] = {5.24, 6.88, 8.59, 10.82, 13.14, 15.33, 19.27, 21.96, 18.78, 16.09, 14.37, 13.17, 11.41, 9.64, 7.98, 6.34, 5.06};
double kernel_s[sample_size_s] = {5.13, 6.39, 8.26, 10.21, 12.02, 11.62, 9.51, 7.32, 5.96, 5.03};
double sub_vector_s[sample_size_s];
double sub_vector_l[sample_size_l];

//----------------OLED Display variables and constants-----------//
SSD1306Wire  display(0x3c, 0, 2);
int displayMode = 0;
int n=0;


const char* ssid = "OnePlus 3T";
const char* password = "password";

//------------------------------IFTTT global variables-----------//
int oneFlag = 0;
int anotherFlag = 0;

//################################################ SETUP #####################################################//
void setup() {

  //----------------------Communication Setup ----------------------//
  Serial.begin(115200);
  Wire.begin(0, 2);
  Wire.setClock(400000L);

  WiFi.begin(ssid, password);

  //---------------------------IMU Setup---------------------------//
  i2cData[0] = 7; // Set the sample rate to 1000Hz - 8kHz/(7+1) = 1000Hz
  i2cData[1] = 0x00; // Disable FSYNC and set 260 Hz Acc filtering, 256 Hz Gyro filtering, 8 KHz sampling
  i2cData[2] = 0x00; // Set Gyro Full Scale Range to ±250deg/s
  i2cData[3] = 0x00; // Set Accelerometer Full Scale Range to ±2g
  while (i2cWrite(IMUAddress, 0x19, i2cData, 4, false)); // Write to all four registers at once
  while (i2cWrite(IMUAddress, 0x6B, 0x01, true)); // PLL with X axis gyroscope reference and disable sleep mode

  while (i2cRead(IMUAddress, 0x75, i2cData, 1));
  if (i2cData[0] != 0x71) { // Read "WHO_AM_I" register
    Serial.print(F("Error reading sensor"));
    while (1);
  }

  delay(1000); // Wait for sensor to stabilize
  Serial.println("calibrating");
  delay(3000);
  calibrateAccelGyro(300);

  //-----------------------ADS 1115 Setup---------------------------//
  ads.begin();
  distancetimer = millis();
  
  //-----------------------OLED Display Setup-----------------------//
  display.init();
  display.flipScreenVertically();
  displaylogo(displayMode);

  //-------------------------WiFi Setup----------------------------//
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print("Connecting..");
  }
}

//########################################## LOOP #########################################//
void loop() {
 
  //-----------------------Read IMU data--------------------//
  while (i2cRead(IMUAddress, 0x3B, i2cData, 14));
  accX = ((i2cData[0] << 8) | i2cData[1]);
  accY = ((i2cData[2] << 8) | i2cData[3]);
  accZ = ((i2cData[4] << 8) | i2cData[5]);

  map(accX, 0, 65535, 0, 1023);
  map(accY, 0, 65535, 0, 1023);
  map(accZ, 0, 65535, 0, 1023);

  accX = accX - delt_aX;
  accY = accY - delt_aY;
  accZ = accZ - delt_aZ;
  
  Pitch = -(atan2(accX, sqrt(accY*accY + accZ*accZ))*180.0)/M_PI;
  Roll  = (atan2(accY, accZ)*180.0)/M_PI;

  sharp_adc = ads.readADC_SingleEnded(0);
  voltage = sharp_adc*0.1875/1000;
  distance = 13*pow(voltage,-1);
  
  
  if ((millis() - distancetimer > timeinterval) && distance > 5 && distance < 25){
    for(int i=sample_size_l - 1; i>0; i--){
      dist_vector[i] = dist_vector[i-1];
      sub_vector_l[i] = dist_vector[i] - kernel_l[i];
      if(i<(sample_size_s)){
        sub_vector_s[i] = dist_vector[i] - kernel_s[i];
      }
    }
    
    dist_vector[0] = distance;
    sub_vector_l[0] = dist_vector[0] - kernel_l[0];
    sub_vector_s[0] = dist_vector[0] - kernel_s[0];
    
    mean_l = mean(sub_vector_l, sample_size_l);
    sd_l = sd(sub_vector_l, sample_size_l, mean_l);
    mean_s = mean(sub_vector_s, sample_size_s);
    sd_s = sd(sub_vector_s, sample_size_s, mean_s);

    if((mean_l < 1.51)&&(mean_l>(-1.51))&&(sd_l<3.1)&&(sd_l>(-3.1))){
      Serial.println("long");
      displayMode += 1;
      displayMode %= 4;
      displaylogo(displayMode);
      delay(1000);
      if(displayMode == 1)
      {
        showmusiclist(0);
      }
    }
    else if((mean_s < 1.01)&&(mean_s>(-1.01))&&(sd_s<1.51)&&(sd_s>(-1.51))){
      Serial.println("short");
      if(displayMode == 1)
      {
        //Make GET request for song selection
        int songNumber = showmusiclist(0);
        //****************************************ADD CODE
        Serial.print("Song Selected : ");
        Serial.println(songNumber);
        showmusiclist(1);
       
      }
      else if(displayMode == 2)
      {
        //Make GET request for SOS
        //*************************************
        Serial.println("Yes-Voh-Yes");
      }
     else if(oneFlag == 1 && displayMode == 3)
     {
      //Make GET request to one thing
      //**************
      Serial.println("Do One Thing");
      oneFlag = 0;
     }
     else if(anotherFlag = 1 && displayMode == 3)
     {
      //MAKE GET request to do another thing
      //*************************************
      Serial.println("Do another Thing");
      IFTTTRequest("http://maker.ifttt.com/trigger/{event}/with/key/mXkwCqtOPljCxb6fUAWp5lSwo0BvBjsqEyX4nVTVB9k");
     }
    }
    if(Roll<49 && Roll>30 && displayMode == 1)
    {
      showmusiclist(1);
    }
    else if(Roll<20 && displayMode == 1)
    {
      showmusiclist(-1); 
    }
    else if(Roll<49 && Roll>30 && displayMode == 3)
    {
      oneFlag = 1;
    }
    else if(Roll<20 && displayMode == 3)
    {
      anotherFlag = 1; 
    }
  }
  distancetimer = millis();
}


//################################################## HELPER FUNCTIONS ##############################################//

//---------------------------I2C Communication Helper Functions-------------------------------//
uint8_t i2cWrite(uint8_t DeviceAddress, uint8_t registerAddress, uint8_t data, bool sendStop) {
  return i2cWrite(DeviceAddress, registerAddress, &data, 1, sendStop); // Returns 0 on success
}

uint8_t i2cWrite(uint8_t DeviceAddress, uint8_t registerAddress, uint8_t *data, uint8_t length, bool sendStop) {
  Wire.beginTransmission(DeviceAddress);
  Wire.write(registerAddress);
  Wire.write(data, length);
  uint8_t rcode = Wire.endTransmission(sendStop); // Returns 0 on success
  if (rcode) {
    Serial.print(F("i2cWrite failed: "));
    Serial.println(rcode);
  }
  return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
}

uint8_t i2cRead(uint8_t DeviceAddress, uint8_t registerAddress, uint8_t *data, uint8_t nbytes) {
  uint32_t timeOutTimer;
  Wire.beginTransmission(DeviceAddress);
  Wire.write(registerAddress);
  uint8_t rcode = Wire.endTransmission(false); // Don't release the bus
  if (rcode) {
    Serial.print(F("i2cRead failed: "));
    Serial.println(rcode);
    return rcode; // See: http://arduino.cc/en/Reference/WireEndTransmission
  }
  Wire.requestFrom(DeviceAddress, nbytes, (uint8_t)true); // Send a repeated start and then release the bus after reading
  for (uint8_t i = 0; i < nbytes; i++) {
    if (Wire.available())
      data[i] = Wire.read();
    else {
      timeOutTimer = micros();
      while (((micros() - timeOutTimer) < I2C_TIMEOUT) && !Wire.available());
      if (Wire.available())
        data[i] = Wire.read();
      else {
        Serial.println(F("i2cRead timeout"));
        return 5; // This error value is not already taken by endTransmission
      }
    }
  }
  return 0; // Success
}

//------------------------------------OLED display helper functions-----------------------------------//
void displaylogo(int val)
{
  switch(val)
  {
    case 0:  //Clock Display
      display.clear();
      display.drawXbm(0, 0, ww, wh, wifi_logo);
      display.setFont(ArialMT_Plain_24);
      display.setTextAlignment(TEXT_ALIGN_CENTER);
      display.drawString(64, 12, "11:12 A.M.");
      display.drawHorizontalLine(0, 42, 128);
      display.setFont(ArialMT_Plain_10);
      display.setTextAlignment(TEXT_ALIGN_CENTER);
      display.drawString(64, 43, "22,Sunday,2018");
      display.display();
      break;
    case 1:  //music logo
      display.clear();
      display.drawXbm(30, 0, width, height, music_logo);
      display.display();
      break;
    case 2:  //sos logo
      display.clear();
      display.drawXbm(30, 0, width, height, sos_logo);
      display.display();
      break;
    case 3:  //xtencion logo
      display.clear();
      display.drawXbm(30, 0, width, height, x_logo);
      display.display();
      break;
  }
}

int showmusiclist(int num)
{   n=n+num;
   switch (num) {
    case 1:
      if(n==3){n=0;}
      display.clear();
      display.drawString(3,n*18,"=>");
      display.setFont(ArialMT_Plain_16);
      display.setTextAlignment(TEXT_ALIGN_LEFT);
      display.drawString(24,00,"Rolex");
      display.drawString(24,18,"Highway to hell");
      display.drawString(24,36,"Iron man");
      display.display();
      //Serial.println(n);
      break;
    case -1:
      if(n==-1){n=2;}
      display.clear();
      display.drawString(3,n*18,"=>");
      display.setFont(ArialMT_Plain_16);
      display.setTextAlignment(TEXT_ALIGN_LEFT);
      display.drawString(24,00,"Rolex");
      display.drawString(24,18,"Highway to hell");
      display.drawString(24,36,"Iron man");
      display.display();
     // Serial.println(n);
      break;
    default:
      display.clear();
      display.drawString(3,n*18,"=>");
      display.setFont(ArialMT_Plain_16);
      display.setTextAlignment(TEXT_ALIGN_LEFT);
      display.drawString(24,00,"Rolex");
      display.drawString(24,18,"Highway to hell");
      display.drawString(24,36,"Iron man");
      display.display();     
  }
  
  return(n);
}

//-----------------------------IFTTT HTTP REQUEST FUNCTION-----------------------------//
void IFTTTRequest(String url)
{
  if (WiFi.status() == WL_CONNECTED) { //Check WiFi connection status
 
    HTTPClient http;  //Declare an object of class HTTPClient
 
    http.begin(url);  //Specify request destination
    int httpCode = http.GET();//Send the request
 
    if (httpCode > 0) { //Check the returning code
 
      String payload = http.getString();   //Get the request response payload
      Serial.println(payload);                     //Print the response payload
 
    }
 
    http.end();   //Close connection
  }
}

//-------------------------------------------Swipe Calculation helper functions-----------------------------------//
double sd(double sub_vector[],double sample_size, double mean){
    double standard_deviation, sum_of_difference, difference;
    sum_of_difference = 0.00;
    for(int i=0; i<sample_size; i++){
      difference = sub_vector[i] - mean;
      sum_of_difference += (difference*difference);
    }
    standard_deviation = sum_of_difference/sample_size;
    return standard_deviation;
}

double mean(double sub_vector[],double sample_size){
    double mean;
    double sum = 0.00;
    for(int i=0; i<sample_size; i++){
      sum += sub_vector[i];
    }
    mean = sum/sample_size;
    return mean;
}

//-------------------------------------------------GYRO Helper Functions-------------------------------------------//
void calibrateAccelGyro(int samples){
  int16_t aX, aY, aZ;
  int16_t raX, raY, raZ;
  for (int i=0; i< samples; i++){
    while (i2cRead(IMUAddress, 0x3B, i2cData, 14));
    raX = ((i2cData[0] << 8) | i2cData[1]);
    raY = ((i2cData[2] << 8) | i2cData[3]);
    raZ = ((i2cData[4] << 8) | i2cData[5]);
    map(raX, 0, 65535, 0, 1023);
    map(raY, 0, 65535, 0, 1023);
    map(raZ, 0, 65535, 0, 1023);
    aX += raX;
    aY += raY;
    aZ += raZ;
    delay(10);
  }
  delt_aX = aX/samples;
  delt_aY = aY/samples;
  delt_aZ = aZ/samples;
}

//-------------------------------------------------------Swipe functions-------------------------------------------//
void longswipe(){
  tiltInhibit ^= true;
}

void shortswipe(){
  tiltInhibit ^= true;
}

